#!/usr/bin/env python

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import TYPE_CHECKING

from appium.common.helper import extract_const_attributes
from appium.common.logger import logger
from appium.protocols.webdriver.can_execute_commands import CanExecuteCommands
from appium.webdriver.mobilecommand import MobileCommand as Command

if TYPE_CHECKING:
    from appium.webdriver.webdriver import WebDriver


class NetSpeed:
    GSM = 'gsm'  # GSM/CSD (up: 14.4(kbps), down: 14.4(kbps))
    SCSD = 'scsd'  # HSCSD (up: 14.4, down: 57.6)
    GPRS = 'gprs'  # GPRS (up: 28.8, down: 57.6)
    EDGE = 'edge'  # EDGE/EGPRS (up: 473.6, down: 473.6)
    UMTS = 'umts'  # UMTS/3G (up: 384.0, down: 384.0)
    HSDPA = 'hsdpa'  # HSDPA (up: 5760.0, down: 13,980.0)
    LTE = 'lte'  # LTE (up: 58,000, down: 173,000)
    EVDO = 'evdo'  # EVDO (up: 75,000, down: 280,000)
    FULL = 'full'  # No limit, the default (up: 0.0, down: 0.0)


class Network(CanExecuteCommands):
    @property
    def network_connection(self) -> int:
        """Returns an integer bitmask specifying the network connection type.

        Android only.
        Possible values are available through the enumeration `appium.webdriver.ConnectionType`
        """
        return self.execute(Command.GET_NETWORK_CONNECTION, {})['value']

    def set_network_connection(self, connection_type: int) -> int:
        """Sets the network connection type. Android only.

        Possible values:

            +--------------------+------+------+---------------+
            | Value (Alias)      | Data | Wifi | Airplane Mode |
            +====================+======+======+===============+
            | 0 (None)           | 0    | 0    | 0             |
            +--------------------+------+------+---------------+
            | 1 (Airplane Mode)  | 0    | 0    | 1             |
            +--------------------+------+------+---------------+
            | 2 (Wifi only)      | 0    | 1    | 0             |
            +--------------------+------+------+---------------+
            | 4 (Data only)      | 1    | 0    | 0             |
            +--------------------+------+------+---------------+
            | 6 (All network on) | 1    | 1    | 0             |
            +--------------------+------+------+---------------+

        These are available through the enumeration `appium.webdriver.ConnectionType`

        Args:
            connection_type: a member of the enum `appium.webdriver.ConnectionType`

        Return:
            int: Set network connection type
        """
        data = {'parameters': {'type': connection_type}}
        return self.execute(Command.SET_NETWORK_CONNECTION, data)['value']

    def toggle_wifi(self) -> 'WebDriver':
        """Toggle the wifi on the device, Android only.

        Returns:
            Union['WebDriver', 'Network']: Self instance
        """
        self.execute(Command.TOGGLE_WIFI, {})
        return self  # type: ignore

    def set_network_speed(self, speed_type: str) -> 'WebDriver':
        """Set the network speed emulation.

        Android Emulator only.

        Args:
            speed_type: The network speed type.
                A member of the const appium.webdriver.extensions.android.network.NetSpeed.

        Usage:
            self.driver.set_network_speed(NetSpeed.LTE)

        Returns:
            Union['WebDriver', 'Network']: Self instance
        """
        constants = extract_const_attributes(NetSpeed)
        if speed_type not in constants.values():
            logger.warning(
                f'{speed_type} is unknown. Consider using one of {list(constants.keys())} constants. '
                f'(e.g. {NetSpeed.__name__}.LTE)'
            )

        self.execute(Command.SET_NETWORK_SPEED, {'netspeed': speed_type})
        return self  # type: ignore

    def _add_commands(self) -> None:
        # noinspection PyProtectedMember,PyUnresolvedReferences
        commands = self.command_executor._commands
        commands[Command.TOGGLE_WIFI] = ('POST', '/session/$sessionId/appium/device/toggle_wifi')
        commands[Command.GET_NETWORK_CONNECTION] = (
            'GET',
            '/session/$sessionId/network_connection',
        )
        commands[Command.SET_NETWORK_CONNECTION] = (
            'POST',
            '/session/$sessionId/network_connection',
        )
        commands[Command.SET_NETWORK_SPEED] = (
            'POST',
            '/session/$sessionId/appium/device/network_speed',
        )



import imp
from typing import Optional, Union

from pyallied.web.print_page_options import PrintOptions
from pyallied.web.webWaits import customwebDriverwait
from pyallied.web.log_type import log_type
from pyallied.web.file_detector import FileDetector
import os
import base64
from PIL import Image
from selenium.webdriver.remote.file_detector import UselessFileDetector
class Browser(customwebDriverwait):
    def __init__(self,driver):
        super().__init__(driver)
        self.driver=driver
        '''
        returns capabilities list
        '''    
    def get_desired_capabilities_Of_driver(self):
        try:
            return self.driver.capabilities
        except Exception as error:
            raise error              

    ''' closes the session''' 
    def close_driver(self):
        try:
            self.driver.close()
        except Exception as error:
            raise error          
    ''' quit the driver ''' 
    def quit_driver(self):
        try:
            self.driver.quit()
        except Exception as error:
            raise error          
    '''Goes one step backward in the driver history.''' 
    def go_back(self):
        try:
            self.driver.back()
        except Exception as error:
            raise error          
    '''Delete all cookies in the scope of the session.'''
    def delete_all_Cookies(self):
        try:
            self.driver.delete_all_cookies()
        except Exception as error:
            raise error                               
    '''Deletes a single cookie with the given name. '''
    def delete_a_Cookie(self,name):
        try:
            self.driver.delete_cookie(name)
        except Exception as error:
            raise error          
    '''
    Asynchronously Executes JavaScript in the current window/frame.

    Args:	
    script: The JavaScript to execute.
    *args: Any applicable arguments for your JavaScript.
    Usage:	
    script = "var callback = arguments[arguments.length - 1]; " \
            "window.setTimeout(function(){ callback('timeout') }, 3000);"
    driver.execute_async_script(script)
    '''
    def Execute_Async_Script(self,script: str, *args):
        try:
            self.driver.execute_async_script(script,*args)
        except Exception as error:
            raise error          
    '''
    Synchronously Executes JavaScript in the current window/frame.

    Args:	
    script: The JavaScript to execute.
    *args: Any applicable arguments for your JavaScript.
    Usage:	
    driver.execute_script('return document.title;')
    ''' 
    def Execute_Script(self,script: str, *args):
        try:
            self.driver.execute_script(script,*args)
        except Exception as error:
            raise error  
    '''
    Sends a command to be executed by a command.CommandExecutor.

    Args:	
    driver_command: The name of the command to execute as a string.
    params: A dictionary of named parameters to send with the command.
    Returns:	
    The command’s JSON response loaded into a dictionary object.
    '''
    def Execute_Command(self,driver_command: str, params: dict = None):
        try:
            self.driver.execute(driver_command,params)
        except Exception as error:
            raise error
    '''
    Overrides the current file detector (if necessary) in limited context. Ensures the original file detector is set afterwards.

    Example:

    with webdriver.file_detector_context(UselessFileDetector):
    someinput.send_keys(‘/etc/hosts’)
    Args:	
    file_detector_class - Class of the desired file detector. If the class is different
    from the current file_detector, then the class is instantiated with args and kwargs and used as a file detector during the duration of the context manager.
    args - Optional arguments that get passed to the file detector class during
    instantiation.
    kwargs - Keyword arguments, passed the same way as args.    
    '''  
    #def File_Detector_Context(self,file_detector_class, *args, **kwargs):
       
    '''
    Goes one step forward in the browser history.

    Usage:	
    driver.forward()
    ''' 
    def Forward(self):
        try:
            self.driver.forward()
        except Exception as error:
            raise error           
    '''
    Invokes the window manager-specific ‘full screen’ operation
    '''
    def FullScreen_Window(self):
        try:
            self.driver.fullscreen_window()
        except Exception as error:
            raise error
    def Get_Cookie(self,name):
        try:
            return self.driver.get_cookie(name)
        except Exception as error:
            raise error
    def Get_Credentials(self):
        try:
            return self.driver.get_credentials()
        except Exception as error:
            raise error 
    def Get_Log(self,log_type):
        try:
            return self.driver.get_log(log_type)
        except Exception as error:
            raise error
    def Get_pinned_Scripts(self):
        try:
            return self.driver.get_pinned_scripts()
        except Exception as error:
            raise error
             
    def Get_page_load_timeout(self):
        try:
            return self.driver.timeouts.page_load
        except Exception as error:
            raise error    
    def Get_implicit_wait_timeout(self):
        try:
            return self.driver.timeouts.implicit_wait
        except Exception as error:
            raise error
    def Get_script_timeout(self):
        try:
            return self.driver.timeouts.script
        except Exception as error:
            raise error
    def Get_Virtual_authenticator_id(self):
        try:
            return self.driver.virtual_authenticator_id
        except Exception as error:
            raise error

    def Get_Title(self):
        try:
            return self.driver.title
        except Exception as error:
            raise error
    def Set_implicitly_wait(self,time_to_wait):
        try:
            self.driver.implicitly_wait(time_to_wait)
        except Exception as error:
            raise error
    def Pin_script(self,script, script_key=None):
        try:
            if(script_key!=None):
                self.driver.pin_script(script,script_key)
            else:
                 self.driver.pin_script(script)   
        except Exception as error:
            raise error
    '''Takes PDF of the current page. The driver makes a best effort to return a PDF based on the provided parameters'''        
    def Print_Page(self,print_options: Optional[PrintOptions] = None):
        try:
            self.driver.print_page(print_options)
        except Exception as error:
            raise error
    def Refresh(self):
        try:
            self.driver.refresh()
        except Exception as error:
            raise error
    '''Removes all credentials from the authenticator.'''        
    def Remove_all_credentials(self):
        try:
            self.driver.remove_all_credentials()
        except Exception as error:
            raise error
    ''' Removes a credential from the authenticator.'''        
    def Remove_credential(self,credential_id: Union[str, bytearray]):
        try:
            self.driver.remove_credential(credential_id)
        except Exception as error:
            raise error
    def Remove_virtual_authenticator(self):
        try:
            self.driver.remove_virtual_authenticator()
        except Exception as error:
            raise error
    '''
        Set the amount of time to wait for a page load to complete
    before throwing an error.
    Args:	
    time_to_wait: The amount of time to wait
    Usage:	
    driver.set_page_load_timeout(30)
    '''        
    def Set_page_load_timeout(self,time_to_wait):
        try:
            self.driver.set_page_load_timeout(time_to_wait) 
        except Exception as error:
            raise error
    '''Set the amount of time that the script should wait during an
    execute_async_script call before throwing an error.
    Args:	
    time_to_wait: The amount of time to wait (in seconds)
    Usage:	
    driver.set_script_timeout(30)
    '''         
    def set_script_timeout(self,time_to_wait):
        try:
            self.driver.set_script_timeout(time_to_wait) 
        except Exception as error:
            raise error
    '''
    Sets whether the authenticator will simulate success or fail on user verification. 
    verified: True if the authenticator will pass user verification, False otherwise.
    '''         
    def Set_user_Verified(self,verified: bool):
        try:
            self.driver.set_user_verified(verified)
        except Exception as error:
            raise error
    '''
    Sets the x,y position of the current window. (window.moveTo)

    Args:	
    x: the x-coordinate in pixels to set the window position
    y: the y-coordinate in pixels to set the window position
    Usage:	
    driver.set_window_position(0,0)
    
    '''        
    def Set_window_position(self,x, y, windowHandle='current'):
        try:
            self.driver.set_window_position(x,y)
        except Exception as error:
            raise error
    '''
    Sets the x, y coordinates of the window as well as height and width of the current window. This method is only supported for W3C compatible browsers; other browsers should use set_window_position and set_window_size.

    Usage:	
    driver.set_window_rect(x=10, y=10)
    driver.set_window_rect(width=100, height=200)
    driver.set_window_rect(x=10, y=10, width=100, height=200)
    '''        
    def Set_window_rect(self,x=None, y=None, width=None, height=None):
        try:
            self.driver.set_window_rect(x,y,width,height)
        except Exception as error:
            raise error 
    '''
    Sets the width and height of the current window. (window.resizeTo)

    Args:	
    width: the width in pixels to set the window to
    height: the height in pixels to set the window to
    Usage:	
    driver.set_window_size(800,600)
    '''
    def Set_window_size(self,width, height, windowHandle='current'):
        try:
            self.driver.set_window_size(width, height)
        except Exception as error:
            raise error
    '''Called before starting a new session. This method may be overridden to define custom startup behavior.'''        
    def Start_client(self):
        try:
            self.driver.start_client()
        except Exception as error:
            raise error
    '''
    Creates a new session with the desired capabilities.

    Args:	
    capabilities - a capabilities dict to start the session with.
    browser_profile - A selenium.webdriver.firefox.firefox_profile.FirefoxProfile object. Only used if Firefox is requested.
    '''        
    def Start_session(self,capabilities: dict, browser_profile=None):
        try:
            if(browser_profile!=None):
                self.driver.start_session(capabilities,browser_profile)
            else:
                self.driver.start_session(capabilities)   
        except Exception as error:
            raise error
    '''Called after executing a quit command. This method may be overridden to define custom shutdown behavior.'''        
    def Stop_client(self):
        try:
            self.driver.stop_client()
        except Exception as error:
            raise error 
    def Unpin(self,script_key):
        try:
            self.driver.unpin(script_key)
        except Exception as error:
            raise error
    '''Returns a ApplicationCache Object to interact with the browser app cache'''        
    def Get_application_cache(self):
        try:
            return self.driver.application_cache
        except Exception as error:
            raise error 
    '''Returns the URL of the current page.'''        
    def Get_current_url(self):
        try:
            return self.driver.current_url
        except Exception as error:
            raise error
    '''Returns the handle of the current window.''' 
    def Get_current_window_handle(self):
        try:
            return self.driver.current_window_handle
        except Exception as error:
            raise error 
    '''returns the drivers current desired capabilities being used'''        
    def Get_desired_capabilities(self):
        try:
            return self.driver.desired_capabilities
        except Exception as error:
            raise error
    def Get_desired_capabilities(self):
        try:
            return self.driver.desired_capabilities
        except Exception as error:
            raise error
    def Set_file_detector(self,detectorType:UselessFileDetector):
        try:
            self.driver.file_detector=detectorType
        except Exception as error:
            raise error
    def Get_Page_Source(self):
        try:
            return self.driver.page_source
        except Exception as error:
            raise error
    def get_command_line_args(self):
        try:
            return self.driver.service.command_line_args()
        except Exception as error:
            raise error 
    def get_assert_process_still_running(self):
        try:
            return self.driver.service.assert_process_still_running()
        except Exception as error:
            raise error
    def get_is_connectable(self):
        try:
            return self.driver.service.is_connectable()
        except Exception as error:
            raise error
    def Send_remote_shutdown_command(self):
        try:
             self.driver.service.send_remote_shutdown_command()
        except Exception as error:
            raise error
    '''
    Starts the Service.

    Exceptions:	
    WebDriverException : Raised either when it can’t start the service or when it can’t connect to the service
    '''         
    def start_service(self):
        try:
             self.driver.service.start()
        except Exception as error:
            raise error
    def stop_service(self):
        try:
             self.driver.service.stop()
        except Exception as error:
            raise error
    def get_service_url(self):
        try:
            return self.driver.service.service_url
        except Exception as error:
            raise error 
    def get_driver_name(self):
        try:
            return self.driver.name
        except Exception as error:
            raise error
    def get_orientation(self):
        try:
            return self.driver.orientation
        except Exception as error:
            raise error
    def get_timeouts(self):
        try:
            return self.driver.timeouts
        except Exception as error:
            raise error
    def get_screenshot_base64(self,name): 
        try:
            with open(self.__Screenshot_Location()+"/"+name+".jpg", "wb") as fh:
                fh.write(base64.urlsafe_b64decode(self.driver.get_screenshot_as_base64()))            
            
        except Exception as error:
            raise error
    def get_screenshot_png(self,name): 
        try:
            with open(self.__Screenshot_Location()+"/"+name+".png", "wb") as fh:
                fh.write(self.driver.get_screenshot_as_png())            
            
        except Exception as error:
            raise error
    def get_screenshot_file(self,name): 
        try:
            self.driver.get_screenshot_as_file(self.__Screenshot_Location()+"/"+name+'.png')
            
        except Exception as error:
            raise error                        
    def __Screenshot_Location(self):
        try:
            dir_name=os.getcwd()+"/page_Screenshots"
            os.makedirs(dir_name, exist_ok=True)
            return dir_name
        except FileExistsError:
            pass                                                                                                                                                                               


from select import select
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver import ActionChains
from selenium.webdriver.common.alert import Alert
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support.select import Select
from selenium.webdriver.chrome import service
from pyallied.web.webWaits import customwebDriverwait
class driverAlerts(customwebDriverwait):

    def __init__(self, driver):
        super().__init__(driver)
        self.driver = driver



    #alerts
    # Use this class to interact with alert prompts. 
    # It contains methods for dismissing, accepting, inputting, and getting text from alert prompts.   
    def acceptAlert(self):
        try:
            alertPresence= super().WaitForAlert_is_present()
            if(alertPresence):
                Alert(self.driver).accept()
        except Exception as error:
            raise error                
    def dismissAlert(self):
        try:
            alertPresence= super().WaitForAlert_is_present()
            if(alertPresence):
                Alert(self.driver).dismiss() 
        except Exception as error:
            raise error                
    def getAlertText(self):
        try:
            alertPresence= super().WaitForAlert_is_present()
            if(alertPresence):
                alertText=Alert(self.driver).text
                return alertText
        except Exception as error:
            raise error                
    def sendKeystoAlert(self,keysToSend):
        try:
            alertPresence= super().WaitForAlert_is_present()
            alert=self.driver.switch_to.alert
            alert.send_keys(keysToSend)
            #name_prompt = Alert(self.driver)
            #name_prompt.send_keys(keysToSend)
        except Exception as error:
            raise error            
    # Special Keys
    #The Keys implementation.
    ''' yet to complete'''
    #Touch Actions
    ''' yet to complete'''









import base64
from distutils.log import error
from typing import Dict,Optional,List,Any
from base64 import b64encode
from xmlrpc.client import boolean
#from selenium.webdriver.common.utils import is_url_connectable
from selenium import webdriver
from appium.webdriver.common.appiumby import AppiumBy
from selenium.webdriver.common import utils
from appium.protocols.webdriver.can_execute_scripts import CanExecuteScripts
import polling2
import time
from appium.webdriver.common.appiumby import AppiumBy
from selenium.common.exceptions import NoSuchElementException
from selenium.common.exceptions import StaleElementReferenceException
from selenium.common.exceptions import ElementNotVisibleException
from appium.webdriver.extensions.keyboard import Keyboard
from pyallied.mobile.logTypes import logTypes
from pyallied.mobile.AndroidPoweOptions import android_poweroptions
from pyallied.mobile.key_event_keyCodes import key_codes
from appium.webdriver.extensions import keyboard
from pyallied.mobile.gsm import GsmCallActions
from pyallied.mobile.gsm import GsmSignalStrength
from pyallied.mobile.gsm import GsmVoiceState
from pyallied.mobile.network import NetSpeed

import textwrap


class mobAppium:
    def __init__(self, driver):
        self.driver = driver
        '''
        Returns information about whether a remote end is in a state in which it can create new sessions 
        and can additionally include arbitrary meta information that is specific to the implementation.
        usage: selenium.webdriver.common.utils.is_url_connectable(port)


        '''

    def get_Remote_Driver_Status(self, port):
        utils.is_url_connectable(port)

        '''
        Execute a variety of native, mobile commands that aren't associated with a specific endpoint

        Syntax is execute("mobile: <commandName>", <JSON serializable argument>)
        
        useage: self.driver.execute_script("mobile: scroll", {'direction': 'down'})


        '''

    def execute_Mobile_Command(self, Mobilecommand: str, parameters: Dict[str, any]):
        self.driver.execute_script(Mobilecommand, parameters)
        '''
        The server should attempt to create a session that most closely matches the desired and required capabilities.

        JSONWP Spec Required capabilities have higher priority than desired 
        capabilities and must be set for the session to be created
        W3C Spec capabilities.alwaysMatch must be set for session to be created; 
        capabilities.firstMatch must match at least one (the first one to match will be used)

        useage:
        desired_caps = {
            'platformName': 'Android',
            'platformVersion': '7.0',
            'deviceName': 'Android Emulator',
            'automationName': 'UiAutomator2',
            'app': PATH('/path/to/app')
            }
        self.driver = webdriver.Remote('http://127.0.0.1:4723/wd/hub', desired_caps)

        '''

    def create_NewSession(self, hostAddress: str, desired_caps: Dict[str, str]):
        self.driver = webdriver.Remote(hostAddress, desired_caps)
        '''
        end the session
        '''

    def end_Session(self):
        self.driver.quit()
        ''' Returns dict of capabilities '''

    def get_Session_Capabilities(self):
        desired_caps = self.driver.session
        ''' 
        Navigate backwards in the driver history, if possible (Web context only)

        '''

    def go_Back(self):
        self.driver.back()
        '''
        Take a screenshot of the current viewport/window/page
        Takes a screenshot of the viewport in a native context (iOS, Android) and 
        takes a screenshot of the window in web context

        Note that some platforms may have settings that prevent screenshots from being taken, 
        for security reason. One such feature is the Android FLAG_SECURE layout parameter


        '''

    def take_screenshot(self):
        screenshotBase64 = self.driver.get_screenshot_as_base64()
        return screenshotBase64

        '''
        Get the current application hierarchy XML (app) or page source (web)
        In a web context, the source returns the source HTML of the current window. 
        In a native context (iOS, Android, etc...) it will return the application hierarchy XML.

        This method is useful for inspecting your application hierarchy and using that to write selectors

        (NOTE: iOS and Android don't have standard ways of defining their application source, 
        so on calls to 'Get Page Source' Appium traverses the app hierarchy and creates an XML document. 
        Thus, getting the source can often be an expensive and time-consuming operation)


        '''

    def get_PageSource(self):
        source = self.driver.page_source
        return source
        '''
        Configure the amount of time that a particular type of operation can execute for before they are aborted


        '''

    def set_PageLoad_Timeout(self, timeout: int):
        self.driver.set_page_load_timeout(timeout)

        '''Set the amount of time, in milliseconds, that asynchronous scripts executed by execute async are 
        permitted to run before they are aborted (Web context only)
        
        '''

    def set_scriptLoad_Timeout(self, timeout: int):
        self.driver.set_script_timeout(timeout)
        '''
        Set the amount of time the driver should wait when searching for elements
        '''

    def set_Implicit_Wait_Timeout(self, timeout: int):
        self.driver.implicitly_wait(5)  # waits 5 seconds
        '''
        Inject a snippet of JavaScript into the page for execution in the context of the currently 
        selected frame (Web context only)
        
        Args:	
        script: The JavaScript to execute.
        *args: Any applicable arguments for your JavaScript.
        Usage:	
        script = "var callback = arguments[arguments.length - 1]; " \
                "window.setTimeout(function(){ callback('timeout') }, 3000);"
        driver.execute_async_script(script
        '''

    def execute_Async_Script(self, script: str, *args):
        self.driver.execute_async_script(script,*args)

        '''
        execute_script(script, *args)¶
        Synchronously Executes JavaScript in the current window/frame.

        Args:	
        script: The JavaScript to execute.
        *args: Any applicable arguments for your JavaScript.
        Usage:	
        driver.execute_script('return document.title;')
        '''

    def execute_Script(self, script: str, *args):
        self.driver.execute_script(script,*args)
        '''
        Get the current device/driver orientation
        '''

    def get_Orientation(self):
        orientation = self.driver.orientation
        return orientation
        '''
        Set the current device/driver orientation
        '''

    def set_Orientation(self, orientation: str):
        self.driver.orientation = orientation

        '''
        Get the current geo location
        '''

    def get_geoLocation(self):
        location = self.driver.location()
        return location

    def find_elementBy(self, locatorType, locator):
        #print('print----',self.__with_findElement(locatorType, locator))
        return self.__with_findElement(locatorType, locator)

    def find_elements_By(self, locatorType: str, locator: any):

        #print (self.__with_findElements(self,locatorType, locator))
        return self.__with_findElements(locatorType, locator)

    def __with_findElements(self, locatorType, locator):

        if locatorType == "XPATH":
            try:
                elements = polling2.poll(lambda: self.driver.find_elements(AppiumBy.XPATH, locator), ignore_exceptions=(
                    ElementNotVisibleException, NoSuchElementException, StaleElementReferenceException), step=0.5, timeout=30)
                #print(elements," elemet is")
                return elements
            except Exception as error:
                #print(error," error is")
                return error
        if locatorType == "ANDROID_UIAUTOMATOR":
            try:
                elements = polling2.poll(lambda: self.driver.find_elements(AppiumBy.ANDROID_UIAUTOMATOR, locator), ignore_exceptions=(
                    ElementNotVisibleException, NoSuchElementException, StaleElementReferenceException), step=0.5, timeout=30)
                return elements
            except Exception as error:
                return error
        if locatorType == "ANDROID_VIEWTAG":
            try:
                elements = polling2.poll(lambda: self.driver.find_elements(AppiumBy.ANDROID_VIEWTAG, locator), ignore_exceptions=(
                    ElementNotVisibleException, NoSuchElementException, StaleElementReferenceException), step=0.5, timeout=30)
                return elements
            except Exception as error:
                return error
        if locatorType == "ANDROID_DATA_MATCHER":
            try:
                elements = polling2.poll(lambda: self.driver.find_elements(AppiumBy.ANDROID_DATA_MATCHER, locator), ignore_exceptions=(
                    ElementNotVisibleException, NoSuchElementException, StaleElementReferenceException), step=0.5, timeout=30)
                return elements
            except Exception as error:
                return error
        if locatorType == "ANDROID_VIEW_MATCHER":
            try:
                elements = polling2.poll(lambda: self.driver.find_elements(AppiumBy.ANDROID_VIEW_MATCHER, locator), ignore_exceptions=(
                    ElementNotVisibleException, NoSuchElementException, StaleElementReferenceException), step=5, timeout=30)
                return elements
            except Exception as error:
                return error
        if locatorType == "ACCESSIBILITY_ID":
            try:
                elements = polling2.poll(lambda: self.driver.find_elements(AppiumBy.ACCESSIBILITY_ID, locator), ignore_exceptions=(
                    ElementNotVisibleException, NoSuchElementException, StaleElementReferenceException), step=0.5, timeout=30)
                return elements
            except Exception as error:
                return error
        if locatorType == "IMAGE":
            try:
                elements = polling2.poll(lambda: self.driver.find_elements(AppiumBy.IMAGE, locator), ignore_exceptions=(
                    ElementNotVisibleException, NoSuchElementException, StaleElementReferenceException), step=0.5, timeout=30)
                return elements
            except Exception as error:
                return error

    def __with_findElement(self, locatorType, locator):

        if locatorType == "XPATH":
            try:
                element = polling2.poll(lambda: self.driver.find_element(AppiumBy.XPATH, locator), ignore_exceptions=(
                    ElementNotVisibleException, NoSuchElementException, StaleElementReferenceException), step=0.5, timeout=30)
                
                return element
            except Exception as error:
                return error
        if locatorType == "ANDROID_UIAUTOMATOR":
            try:
                element = polling2.poll(lambda: self.driver.find_element(AppiumBy.ANDROID_UIAUTOMATOR, locator), ignore_exceptions=(
                    ElementNotVisibleException, NoSuchElementException, StaleElementReferenceException), step=0.5, timeout=30)
                return element
            except Exception as error:
                return error
        if locatorType == "ANDROID_VIEWTAG":
            try:
                element = polling2.poll(lambda: self.driver.find_element(AppiumBy.ANDROID_VIEWTAG, locator), ignore_exceptions=(
                    ElementNotVisibleException, NoSuchElementException, StaleElementReferenceException), step=0.5, timeout=30)
                return element
            except Exception as error:
                return error
        if locatorType == "ANDROID_DATA_MATCHER":
            try:
                element = polling2.poll(lambda: self.driver.find_element(AppiumBy.ANDROID_DATA_MATCHER, locator), ignore_exceptions=(
                    ElementNotVisibleException, NoSuchElementException, StaleElementReferenceException), step=0.5, timeout=30)
                return element
            except Exception as error:
                return error
        if locatorType == "ANDROID_VIEW_MATCHER":
            try:
                element = polling2.poll(lambda: self.driver.find_element(AppiumBy.ANDROID_VIEW_MATCHER, locator), ignore_exceptions=(
                    ElementNotVisibleException, NoSuchElementException, StaleElementReferenceException), step=5, timeout=30)
                return element
            except Exception as error:
                return error
        if locatorType == "ACCESSIBILITY_ID":
            try:
                element = polling2.poll(lambda: self.driver.find_element(AppiumBy.ACCESSIBILITY_ID, locator), ignore_exceptions=(
                    ElementNotVisibleException, NoSuchElementException, StaleElementReferenceException), step=0.5, timeout=30)
                return element
            except Exception as error:
                return error
        if locatorType == "IMAGE":
            try:
                element = polling2.poll(lambda: self.driver.find_element(AppiumBy.IMAGE, locator), ignore_exceptions=(
                    ElementNotVisibleException, NoSuchElementException, StaleElementReferenceException), step=0.5, timeout=30)
                return element
            except Exception as error:
                return error
    def getLog_Types(self):
        try:
            log_types = self.driver.log_types
            return log_types
        except Exception as error:
            raise error    
            
    '''Get the log for a given log type. Log buffer is reset after each request

    '''            
    def getLogs(self,logType:logTypes):
        try:
            logs = self.driver.get_log(logType)
            return error
        except Exception as error:
            raise error 
    '''
    Log a custom event on the Appium server. (Since Appium 1.16.0)

    Parameters
    vendor – The vendor to log

    event – The event to log

    Usage:
    driver.log_event(‘appium’, ‘funEvent’)

    Returns
    Self instance

    Return type
    Union[‘WebDriver’, ‘LogEvent’]
    '''        
    def Log_Event(self,vendor: str, event: str):
        try:
            self.driver.logEvent(vendor, event)  
        except Exception as error:
            raise error
    '''
    Retrieves events information from the current session (Since Appium 1.16.0)

    Parameters
    type – The event type to filter with
    
    '''        
    def Get_Event(self,type: Optional[List[str]] = None):
        try:
            self.driver.logEvent(type)  
        except Exception as error:
            raise error
    '''
    Set settings for the current session.

    For more on settings, see: https://github.com/appium/appium/blob/master/docs/en/advanced-concepts/settings.md

    Parameters
    settings – dictionary of settings to apply to the current test session
    '''                    
    def Update_Settings(self,settings: Dict[str, Any]):
        try:
            self.driver.update_settings(settings)  
        except Exception as error:
            raise error 
    '''
    Returns the appium server Settings for the current session.

    Do not get Settings confused with Desired Capabilities, they are separate concepts. 
    See https://github.com/appium/appium/blob/master/docs/en/advanced-concepts/settings.md

    Returns
    Current settings
    '''        
    def Get_settings(self):
        try:
            self.driver.get_settings()  
        except Exception as error:
            raise error
    '''
            execute_driver(script: str, script_type: str = 'webdriverio', timeout_ms: Optional[int] = None)→ Any
        Run a set of script against the current session, allowing execution of many commands in one Appium request. Please read http://appium.io/docs/en/commands/session/execute-driver for more details about the acceptable scripts and the output format.

        Parameters
        script – The string consisting of the script itself

        script_type – The name of the script type. Defaults to ‘webdriverio’.

        timeout_ms – The number of ms Appium should wait for the script to finish before killing it due to timeout_ms.

        Usage:
        self.driver.execute_driver(script=’return [];’)
        self.driver.execute_driver(script=’return [];’, script_type=’webdriverio’)
        self.driver.execute_driver(script=’return [];’, script_type=’webdriverio’, timeout_ms=10000)
        Returns
        The result of the script. It has ‘result’ and ‘logs’ keys.

        Return type
        ExecuteDriver.Result

        Raises
        WebDriverException – If something error happenes in the script. The message has the original error message.
    
    '''        
    def execute_Driver(self,script: str, script_type: str = 'webdriverio', timeout_inms: Optional[int] = None):
        try:
            if(timeout_inms!=None):
                self.driver.execute_driver(script=textwrap.dedent(script),script_type='webdriverio',timeout_ms=timeout_inms)
            else:
                self.driver.execute_driver(script=textwrap.dedent(script),script_type='webdriverio')     
        except Exception as error:
            raise error
    def install_app(self,apkPath:str):
        try:
            self.driver.install_app(apkPath)
        except Exception as error:
            raise error 
    def isApp_installed(self, AppName:str):
        try:
            self.driver.is_app_installed(AppName)
        except Exception as error:
            raise error  
    def Launch_App(self):
        try:
            self.driver.launch_app()
        except Exception as error:
            raise error
    '''
    Description
    Send the currently active app to the background, and either return after a certain amount of time, 
    or leave the app deactivated. There are 3 types of parameters which may be passed to this method:

    An object that looks like {"seconds": secs}, where secs is an integer designating how long, in seconds, 
    to background the app for. -1 means to deactivate the app entirely.

    null, which means to deactivate the app entirely.

    Deprecated An integer: how long, in seconds, to background the app for. 
    -1 means to deactivate the app entirely.
    iOS tests with XCUITest can also use the mobile: terminateApp method to 
    terminate the current app (see detailed documentation), and the mobile: 
    activateApp to activate an existing application on the device under test
    and moves it to the foreground (see detailed documentation).

    '''
    def sendApp_To_Background(self):
        try:
            self.driver.background_app(10)
        except Exception as error:
            raise error  
    def close_App(self):
        try:
            self.driver.close_app()
        except Exception as error:
            raise error
    def reset_App(self):
        try:
            self.driver.reset()
        except Exception as error:
            raise error
    def RemoveApp(self):
        try:
            self.driver.remove_app('com.example.AppName');

        except Exception as error:
            raise error
    def activate_App(self,app:str):
        try:
            self.driver.activate_app(app)
            #self.driver.activate_app('io.appium.android.apis') 
        except Exception as error:
            raise error
    def Terminate_App(self, app:str):
        try:
            self.driver.terminate_app('com.apple.Preferences')
        except Exception as error:
            raise error
    '''iOS tests with XCUITest can also use the mobile: queryAppState method. See detailed documentation.

    '''
    def get_App_state(self,state:str):
        try:
            state_app=self.driver.query_app_state(state)
            return state_app
        except Exception as error:
            raise error
    def get_app_Strings(self):
        try:
            appStrings = self.driver.app_strings("en", "/path/to/file")
            return appStrings
        except Exception as error:
            raise error 
    def End_Test_Coverage(self,intent:str, path:str):
        try:
            self.driver.end_test_coverage(intent, path)

        except Exception as error:
            raise error
    '''Get the content of the system clipboard

    '''
    def get_Clipboard(self):
        try:
            self.driver.get_clipboard()
        except Exception as error:
            raise error
    def get_ClipboardTest(self):
        try:
            self.driver.get_clipboard_text()
        except Exception as error:
            raise error
    def set_ClipBoard(self,textToset:str):
        try:
            self.driver.set_clipboard(textToset)
        except Exception as error:
            raise error
    def set_Clipboard_Text(self,TextToSet:str):
        try:
            self.driver.set_clipboard_text(TextToSet)
        except Exception as error:
            raise error 
    '''
    For Android emulator. To set the state of the battery charger to connected or not.

    '''
    def Set_Power_ac_forEmulator(self,powerMode:android_poweroptions):
        try:
            self.driver.set_power_ac(powerMode)
        except Exception as error:
            raise error 
    '''Emulate power capacity change on the connected emulator.

    '''
    def set_power_capacity_forEmulator(self,percentage:int):
        try:
            self.driver.set_power_capacity(50)
        except Exception as error:
            raise error
    '''Place a file onto the device in a particular place

    '''        
    def push_file_To_Device_From_Location(self,destinationPath:str,data:str):
        try:
            self.driver.push_file(destinationPath, base64.b64encode(data).decode('utf-8'))
        except Exception as error:
            raise error    
                      
    '''Retrieve a file from the device's file system


    '''        
    def pull_file_from_Device(self,filePathInDevice:str):
        try:
            filein_base64=self.driver.pull_file(filePathInDevice)
            return filein_base64

        except Exception as error:
            raise error
    '''Retrieve a folder from the device's file system

    ''' 
    def pull_folder_from_Device(self,filePathInDevice:str):
        try:
            folderin_base64=self.driver.pull_folder(filePathInDevice)
            return folderin_base64

        except Exception as error:
            raise error
    def shake_device(self):
        try:
            self.driver.shake()
        except Exception as error:
            raise error
    def lock_device(self):
        try:
            self.driver.lock()
        except Exception as error:
            raise error
    def unlock_device(self):
        try:
            self.driver.unlock()
        except Exception as error:
            raise error
    def isDevice_Locked(self):
        try:
            self.driver.is_locked()
        except Exception as error:
            raise error 
    '''See https://developer.android.com/reference/android/view/KeyEvent.html for reference of available Android key code values
    Sends a keycode to the device.

        Android only. Possible keycodes can be found in http://developer.android.com/reference/android/view/KeyEvent.html.

        Parameters
        keycode – the keycode to be sent to the device

        metastate – meta information about the keycode being sent

        flags – the set of key event flags

        Returns
        Self instance

        Return type
        Union[‘WebDriver’, ‘Keyboard’]
    
    '''          
    def press_Key_code(self,keycode: int, metastate: Optional[int] = None, flags: Optional[int] = None):
        try:
            #print('---')
            self.driver.press_keycode(keycode,metastate,flags)

        except Exception as error:
            raise error
    '''
    Sends a long press of keycode to the device.

    Android only. Possible keycodes can be found in http://developer.android.com/reference/android/view/KeyEvent.html.

    Parameters
    keycode – the keycode to be sent to the device

    metastate – meta information about the keycode being sent

    flags – the set of key event flags

    Returns
    Self instance

    Return type
    Union[‘WebDriver’, ‘Keyboard’]
    '''        
    def Logng_press_Key_code(self,keycode: int, metastate: Optional[int] = None, flags: Optional[int] = None):
        try:
            self.driver.long_press_keycode(keycode,metastate,flags)

        except Exception as error:
            raise error 
    '''
    Hides the software keyboard on the device.

    In iOS, use key_name to press a particular key, or strategy. In Android, no parameters are used.

    Parameters
    key_name – key to press

    key –

    strategy – strategy for closing the keyboard (e.g., tapOutside)

    Returns
    Self instance

    Return type
    Union[‘WebDriver’, ‘Keyboard’]
    '''
    def Hide_Key_Board(self,key_name: Optional[str] = None, key: Optional[str] = None, strategy: Optional[str] = None):
        try:
            self.driver.hide_keyboard(key_name,key,strategy)
        except Exception as error:
            raise error 
    '''
    Attempts to detect whether a software keyboard is present

    Returns
    True if keyboard is shown
    '''
    def is_keyboard_Shown(self):
        try:
            self.driver.is_keyboard_shown()
        except Exception as error:
            raise error
    '''
    Sends a keycode to the device.

    Android only. Possible keycodes can be found in http://developer.android.com/reference/android/view/KeyEvent.html.

    Parameters
    keycode – the keycode to be sent to the device

    metastate – meta information about the keycode being sent

    Returns
    Self instance

    Return type
    Union[‘WebDriver’, ‘Keyboard’]
    '''
    def key_Event(self,keycode: key_codes, metastate: Optional[int] = None):
        try:
            self.driver.keyevent(keycode,metastate)
        except Exception as error:
            raise error                           
    '''
    Switch the state of the WiFi service. Since Android Q, a method to change the WiFi service state has been restricted. 
    #12327 Please toggle the state via UI instead of this method. The UI flow depends on devices. 
    # Please make sure to encode the correct UI flow on your target device under test
    '''
    def toggle_wifi_Service(self):
        try:
            self.driver.toggle_wifi()
        except Exception as error:
            raise error    
    '''
    Switch the state of the location service
    ''' 
    def toggle_location_Services(self):
        try:
            self.driver.toggle_location_services()
        except Exception as error:
            raise error              

    '''
    Simulate an SMS message (Emulator only)
    ''' 
    def send_sms_emulatorOnly(self,number:str,message:str):
        try:
            self.driver.send_sms(number, message)
        except Exception as error:
            raise error    
    '''
    Make GSM call (Emulator only)
    '''
    def make_GSM_call_emulatorOnly(self, number:str,callActions:GsmCallActions ):
        try:
            self.driver.makeGsmCall("5551234567", callActions) 
        except Exception as error:
            raise error    
    '''Set GSM signal strength (Emulator only)

    ''' 
    def set_GSM_signal_strength_emulatorOnly(self,signalStrength:GsmSignalStrength):
        try:
            self.driver.set_gsm_signal(signalStrength) 
        except Exception as error:
            raise error 
    '''Set GSM voice state (Emulator only)

    '''        
    def set_GSM_signal_emulatorOnly(self,gsmVoice:GsmVoiceState):
        try:
            self.driver.set_gsm_voice(gsmVoice)
        except Exception as error:
            raise error 
    '''Set network speed (Emulator only)
    '''                            
    def set_network_speed_emulatorOnly(self,netSpeed:NetSpeed):
        try:
            self.driver.set_network_speed(netSpeed)
        except Exception as error:
            raise error 
    '''
    Returns the information of the system state which is supported to read as like cpu, memory, network traffic, and battery
    ex: self.driver.get_performance_data(‘my.app.package’, ‘cpuinfo’, 5)
    Android only.

    Parameters
    package_name – The package name of the application

    data_type – The type of system state which wants to read. It should be one of the supported performance data types. Check get_performance_data_types() for supported types

    data_read_timeout – The number of attempts to read
    '''        
    def get_performance_Data(self,package_name: str, data_type: str, data_read_timeout: Optional[int] = None):
        try:
           return self.driver.get_performance_data(package_name,data_type,data_read_timeout)
        except Exception as error:
            raise error
    '''Returns the information types of the system state which is supported to read as like cpu, memory, network traffic, and battery.
    . 
    '''         
    def get_performance_data_Types(self):
        try:
           return self.driver.get_performance_data_types()
        except Exception as error:
            raise error  
    '''Start recording screen

    '''              
    def start_recording_Screen(self):
        try:
            self.driver.start_recording_screen()
        except Exception as error:
            raise error
    '''Stop recording screen

    '''        
    def stop_recording_Screen(self):
        try:
            self.driver.stop_recording_screen()
        except Exception as error:
            raise error 
    '''Simulate touchId on iOS Simulator

    Parameters
    match – Simulates a successful touch (True) or a failed touch (False)

    Returns
    Self instance

    Return type
    Union[‘WebDriver’, ‘HardwareActions’]
    '''  
    def touch_id_ios_simulator(self,match:boolean):
        try:
            self.driver.touch_id(match)
        except Exception as error:
            raise error
    '''Toggle enroll touchId on iOS Simulator

    Returns
    Self instance

    Return type
    Union[‘WebDriver’, ‘HardwareActions’]
    '''        
    def toggle_touch_id_enrollment_ios_simulator(self):
        try:
            self.driver.toggle_touch_id_enrollment()
        except Exception as error:
            raise error        
    '''Open Android notifications (Emulator only)'''
    def open_notifications_andriod_emulatorOnly(self):
        try:
            self.driver.open_notifications()
        except Exception as error:
            raise error
    '''Retrieve visibility and bounds information of the status and navigation bars
        

        Android only.

        Returns
        A dictionary whose keys are
        statusBar
        visible

        x

        y

        width

        height

        navigationBar
        visible

        x

        y

        width

        height
    '''
    def get_system_Bars(self):        
        try:
            return self.driver.get_system_bars()
        except Exception as error:
            raise error 
    '''Returns the date and time from the device.

        Returns
        The date and time

        Return type
        str
    '''        
    def get_device_Time(self):        
        try:
            return self.driver.device_time
        except Exception as error:
            raise error                                  
    def get_device_Time(self,format: Optional[str] = None):        
        try:
            
            return self.driver.device_time(format)
        except Exception as error:
            raise error 
    '''Retrieve display density(dpi) of the Android device
    Get the display density, Android only

        Returns
        The display density of the Android device(dpi)

        Usage:
        self.driver.get_display_density()

        Returns
        The display density

        Return type
        int
    '''
    def get_display_density_android(self):        
        try:
            
            return self.driver.get_display_density()
        except Exception as error:
            raise error              
    '''Authenticate users by using their finger print scans on supported emulators.
        Authenticate users by using their finger print scans on supported Android emulators.

        Parameters
        finger_id – Finger prints stored in Android Keystore system (from 1 to 10)

        Returns
        TODO
    '''                                  
    def finger_print_android_emulator(self,finger_id: int):        
        try:
            
            return self.driver.finger_print(finger_id)
        except Exception as error:
            raise error                              




#from tkinter import COMMAND
from typing import Union
#from click import command
from pyallied.mobile.pollWait import pollWait
from appium import webdriver 
from pyallied.mobile.mobilecommand import mobile_command
from appium.webdriver.applicationstate import ApplicationState
from appium.webdriver.extensions.device_time import DeviceTime
from appium.webdriver.extensions.location import Location
from appium.webdriver.extensions.settings import Settings
from appium.protocols.webdriver.can_execute_commands import CanExecuteCommands
from appium.webdriver.applicationstate import ApplicationState
from appium.webdriver.mobilecommand import MobileCommand
from appium.webdriver.appium_connection import AppiumConnection
from selenium.webdriver.remote.switch_to import SwitchTo
class session(pollWait,Location,Settings,CanExecuteCommands,AppiumConnection,MobileCommand,ApplicationState):
    
    def __init__(self, driver):
        super().__init__(driver)
        self.driver = driver
    def page_source(self):
        self.html=self.driver.page_source
        #print(html)
        return self.html
    def is_ServiceRunning(self):
        return self.driver.AppiumConnection.get_timeout()
    def get_Logs(self,type):
        return self.driver.get_log(type)
    def get_WindowRect(self):
        return self.driver.get_window_rect()
    def get_DesiredCaps(self):
        return self.driver.session
    def goBack(self):
        return self.driver.back()
    def take_Screenshot(self):
        screenshotBase64 = self.driver.get_screenshot_as_base64()
        return screenshotBase64
    def get_Orientation(self):
        return self.driver.orientation    
    def set_Orientation(self,orientation):
        self.driver.orientation = str(orientation)
    def get_GeoLocation(self):
        return self.driver.location

        #locationData=Location.location
        #return locationData
    def set_GeoLocation(self, latitude: Union[float, str], longitude: Union[float, str], altitude: Union[float, str] = None, speed: Union[float, str] = None, satellites: Union[float, str] = None):       
        self.driver.set_location(latitude, longitude, altitude)
    def get_Events(self):
        return self.driver.get_events()
    def get_AppStrings(self):
        return self.driver.app_strings()
    def get_DeviceTime(self):
        return self.driver.get_device_time()
    def get_Settings(self):
        return self.driver.get_settings()    
    def get_AvailableIMEengines(self):
        return self.driver.available_ime_engines
    def is_AppInstalled(self):
        return self.driver.is_app_installed()
    def get_battery_info(self):
        return self.driver.battery_info
    def switch_Context(self):
        self.driver.switch_to.context('WEBVIEW_1')
    def get_Contexts(self):
        return self.driver.contexts
    def Execute_driver_to_Get_Properties(self,command:mobile_command):
        return self.driver.execute(command)        

    

            


         




class Command:
    ADD_COOKIE = 'addCookie'
    ADD_CREDENTIAL = 'addCredential'
    ADD_VIRTUAL_AUTHENTICATOR = 'addVirtualAuthenticator'
    CLEAR_ELEMENT = 'clearElement'
    CLICK_ELEMENT = 'clickElement'
    CLOSE = 'close'
    CONTEXT_HANDLES = 'getContextHandles'
    CURRENT_CONTEXT_HANDLE = 'getCurrentContextHandle'
    DELETE_ALL_COOKIES = 'deleteAllCookies'
    DELETE_COOKIE = 'deleteCookie'
    DELETE_SESSION = 'deleteSession'
    ELEMENT_SCREENSHOT = 'elementScreenshot'
    EXECUTE_ASYNC_SCRIPT = 'executeAsyncScript'
    FIND_CHILD_ELEMENT = 'findChildElement'
    FIND_CHILD_ELEMENTS = 'findChildElements'
    FIND_ELEMENT = 'findElement'
    FIND_ELEMENTS = 'findElements'
    FIND_ELEMENTS_FROM_SHADOW_ROOT = 'findElementsFromShadowRoot'
    FIND_ELEMENT_FROM_SHADOW_ROOT = 'findElementFromShadowRoot'
    FULLSCREEN_WINDOW = 'fullscreenWindow'
    GET = 'get'
    GET_ALL_COOKIES = 'getCookies'
    GET_AVAILABLE_LOG_TYPES = 'getAvailableLogTypes'
    GET_COOKIE = 'getCookie'
    GET_CREDENTIALS = 'getCredentials'
    GET_CURRENT_URL = 'getCurrentUrl'
    GET_ELEMENT_ARIA_LABEL = 'getElementAriaLabel'
    GET_ELEMENT_ARIA_ROLE = 'getElementAriaRole'
    GET_ELEMENT_ATTRIBUTE = 'getElementAttribute'
    GET_ELEMENT_PROPERTY = 'getElementProperty'
    GET_ELEMENT_RECT = 'getElementRect'
    GET_ELEMENT_TAG_NAME = 'getElementTagName'
    GET_ELEMENT_TEXT = 'getElementText'
    GET_ELEMENT_VALUE_OF_CSS_PROPERTY = 'getElementValueOfCssProperty'
    GET_LOG = 'getLog'
    GET_NETWORK_CONNECTION = 'getNetworkConnection'
    GET_PAGE_SOURCE = 'getPageSource'
    GET_SCREEN_ORIENTATION = 'getScreenOrientation'
    GET_SHADOW_ROOT = 'getShadowRoot'
    GET_TIMEOUTS = 'getTimeouts'
    GET_TITLE = 'getTitle'
    GET_WINDOW_RECT = 'getWindowRect'
    GO_BACK = 'goBack'
    GO_FORWARD = 'goForward'
    IS_ELEMENT_ENABLED = 'isElementEnabled'
    IS_ELEMENT_SELECTED = 'isElementSelected'
    MINIMIZE_WINDOW = 'minimizeWindow'
    NEW_SESSION = 'newSession'
    NEW_WINDOW = 'newWindow'
    PRINT_PAGE = 'printPage'
    QUIT = 'quit'
    REFRESH = 'refresh'
    REMOVE_ALL_CREDENTIALS = 'removeAllCredentials'
    REMOVE_CREDENTIAL = 'removeCredential'
    REMOVE_VIRTUAL_AUTHENTICATOR = 'removeVirtualAuthenticator'
    SCREENSHOT = 'screenshot'
    SEND_KEYS_TO_ELEMENT = 'sendKeysToElement'
    SET_NETWORK_CONNECTION = 'setNetworkConnection'
    SET_SCREEN_ORIENTATION = 'setScreenOrientation'
    SET_TIMEOUTS = 'setTimeouts'
    SET_USER_VERIFIED = 'setUserVerified'
    SET_WINDOW_RECT = 'setWindowRect'
    SWITCH_TO_CONTEXT = 'switchToContext'
    SWITCH_TO_FRAME = 'switchToFrame'
    SWITCH_TO_PARENT_FRAME = 'switchToParentFrame'
    SWITCH_TO_WINDOW = 'switchToWindow'
    UPLOAD_FILE = 'uploadFile'
    W3C_ACCEPT_ALERT = 'w3cAcceptAlert'
    W3C_ACTIONS = 'actions'
    W3C_CLEAR_ACTIONS = 'clearActionState'
    W3C_DISMISS_ALERT = 'w3cDismissAlert'
    W3C_EXECUTE_SCRIPT = 'w3cExecuteScript'
    W3C_EXECUTE_SCRIPT_ASYNC = 'w3cExecuteScriptAsync'
    W3C_GET_ACTIVE_ELEMENT = 'w3cGetActiveElement'
    W3C_GET_ALERT_TEXT = 'w3cGetAlertText'
    W3C_GET_CURRENT_WINDOW_HANDLE = 'w3cGetCurrentWindowHandle'
    W3C_GET_WINDOW_HANDLES = 'w3cGetWindowHandles'
    W3C_MAXIMIZE_WINDOW = 'w3cMaximizeWindow'
    W3C_SET_ALERT_VALUE = 'w3cSetAlertValue'


import base64
from distutils.log import error
from typing import Dict,Optional,List,Any
from base64 import b64encode
from xmlrpc.client import boolean
from selenium.webdriver.common.utils import is_url_connectable
from selenium import webdriver
from appium.webdriver.common.appiumby import AppiumBy
from appium.protocols.webdriver.can_execute_scripts import CanExecuteScripts
import polling2
import time
from appium.webdriver.common.appiumby import AppiumBy
from selenium.common.exceptions import NoSuchElementException
from selenium.common.exceptions import StaleElementReferenceException
from selenium.common.exceptions import ElementNotVisibleException
from selenium.webdriver import ActionChains
from pyallied.mobile.logTypes import logTypes
from pyallied.mobile.AndroidPoweOptions import android_poweroptions
from pyallied.mobile.key_event_keyCodes import key_codes
from appium.webdriver.extensions import keyboard
from pyallied.mobile.gsm import GsmCallActions
from pyallied.mobile.gsm import GsmSignalStrength
from pyallied.mobile.gsm import GsmVoiceState
from pyallied.mobile.network import NetSpeed
from pyallied.mobile.mobAppium import mobAppium
from appium.webdriver.common.touch_action import TouchAction
from appium.webdriver.extensions.action_helpers import ActionHelpers
from appium.webdriver.common.multi_action import MultiAction

import textwrap


class mobAppium_actions(mobAppium):
    def __init__(self, driver):
        super().__init__(driver)
        self.driver = driver
    def click(self,locatorType, locator):
        try:
            element=super().find_elementBy(locatorType, locator)
            element.click()
        except Exception as error:
            raise error              
    '''Simulates typing into the element.

    Parameters
    value – A string for typing.

    Returns
    appium.webdriver.webelement.WebElement
    '''
    def send_Keys(self,locatorType, locator,*value: str):
        try:
            element=super().find_elementBy(locatorType, locator)
            element.send_keys(*value)
        except Exception as error:
            raise error                
    '''Clears text.

    Override for Appium

    Returns
    appium.webdriver.webelement.WebElement
    '''          
    def clear(self,locatorType, locator):
        try:
            element=super().find_elementBy(locatorType, locator)
            element.clear()
        except Exception as error:
            raise error           
    def get_text(self,locatortype,locator):
        try:
            element=super().find_elementBy(locatortype, locator)
            return element.text
        except Exception as error:
            raise error
    '''Get an element's tag name

    '''            
    def get_tagName(self,locatortype,locator):
        try:
            element=super().find_elementBy(locatortype, locator)
            return element.tag_name
        except Exception as error:
            raise error 
    '''Get the value of an element's attribute
    '''            
    def get_attribute(self,locatortype,locator,attributeName:str):
        try:
            element=super().find_elementBy(locatortype, locator)
            return element.get_attribute(attributeName)
        except Exception as error:
            raise error 
    '''Determine if a form or form-like element (checkbox, select, etc...) is selected
    '''            
    def is_Selected(self,locatortype,locator):
        try:
            element=super().find_elementBy(locatortype, locator)
            return element.is_selected()
        except Exception as error:
            raise error 
    '''Determine if an element is currently enabled
    '''
    def is_Enabled(self,locatortype,locator):
        try:
            element=super().find_elementBy(locatortype, locator)
            return element.is_enabled()
        except Exception as error:
            raise error 
    '''Determine if an element is currently displayed
    '''
    def is_Displayed(self,locatortype,locator):
        try:
            element=super().find_elementBy(locatortype, locator)
            return element.is_displayed()
        except Exception as error:
            raise error
    '''propertylocation: Dict[str, float]
        Retrieves the current location
        Returns
        A dictionary whose keys are
        latitude (float)

        longitude (float)

        altitude (float)
    '''
    def get_location(self,locatortype,locator):
        try:
            element=super().find_elementBy(locatortype, locator)
            return element.location
        except Exception as error:
            raise error  
    '''Determine an element's size in pixels''' 
    def get_Element_size_inPixel(self,locatortype,locator):
        try:
            element=super().find_elementBy(locatortype, locator)
            return element.size
        except Exception as error:
            raise error 
    '''Gets dimensions and coordinates of an element
    ''' 
    def get_Element_dimensions_coordinates(self,locatortype,locator):
        try:
            element=super().find_elementBy(locatortype, locator)
            return element.rect
        except Exception as error:
            raise error 
    '''Query the value of a web element's computed CSS property
    ''' 
    def get_value_of_css_property(self,locatortype,locator,cssProperty:str):
        try:
            element=super().find_elementBy(locatortype, locator)
            return element.value_of_css_property(cssProperty)
        except Exception as error:
            raise error
    '''Determine an element's location on the screen once it has been scrolled into view (mainly an internal command and not supported by all clients)
    Gets the location of an element relative to the view.

    Usage:
    location = element.location_in_view
    x = location[‘x’]
    y = location[‘y’]
    Returns
    The location of an element relative to the view

    Return type
    dict
    
    '''
    def get_location_in_view(self,locatortype,locator):
        try:
            element=super().find_elementBy(locatortype, locator)
            return element.location_in_view
        except Exception as error:
            raise error 
    '''Submit a FORM element'''
    def submit_Form_element(self,locatortype,locator):
        try:
            element=super().find_elementBy(locatortype, locator)
            return element.submit()
        except Exception as error:
            raise error 
    '''Gets the active element of the current session'''
    def get_active_element(self):
        try:
            element=self.driver.switch_to.active_element
            return element
        except Exception as error:
            raise error             
    '''Move the mouse by an offset of the specificed element'''
    def Move_the_mouse_by_an_offset(self,locatortype, locator,x_offset:int,y_offset:int):
        try:
            element=super().find_elementBy(locatortype, locator)
            actions = ActionChains(self.driver)
            actions.move_to_element_with_offset(element, x_offset, y_offset)
            actions.perform()
        except Exception as error:
            raise error
    '''Click any mouse button at the current mouse coordinates
    '''        
    def Move_the_mouse_toEleement_click(self,locatortype, locator):
        try:
            element=super().find_elementBy(locatortype, locator)
            actions = ActionChains(self.driver)
            actions.move_to_element(element)
            actions.click()
            actions.perform()
        except Exception as error:
            raise error 
    '''Click any mouse button at the current mouse coordinates
    '''        
    def Move_the_mouse_toEleement_SubElement_click(self,locatortype, locator_1,locator_2):
        try:
            element_1=super().find_elementBy(locatortype, locator_1)
            element_2=super().find_elementBy(locatortype, locator_2)
            actions = ActionChains(self.driver)
            actions.move_to_element(element_1)
            actions.click(element_2)
            actions.perform()
        except Exception as error:
            raise error             
    '''Double-clicks at the current mouse coordinates (set by moveto).
    '''                   
    def Move_the_mouse_toEleement_double_click(self,locatortype, locator):
        try:
            element=super().find_elementBy(locatortype, locator)
            actions = ActionChains(self.driver)
            actions.move_to_element(element)
            actions.double_click()
            actions.perform()
        except Exception as error:
            raise error 
    '''Click and hold the left mouse button at the current mouse coordinates
    '''
    def Move_the_mouse_toEleement_click_and_hold(self,locatortype, locator):
        try:
            element=super().find_elementBy(locatortype, locator)
            actions = ActionChains(self.driver)
            actions.move_to_element(element)
            actions.click_and_hold()
            actions.perform()
        except Exception as error:
            raise error
    '''Releases the mouse button previously held'''
    def Move_the_mouse_toEleement_release(self,locatortype, locator):
        try:
            element=super().find_elementBy(locatortype, locator)
            actions = ActionChains(self.driver)
            actions.move_to_element(element)
            actions.click_and_hold()
            actions.move_to_element(element)
            actions.release()
            actions.perform()
        except Exception as error:
            raise error
    '''Single tap on the touch enabled device''' 
    def single_tap_element(self,locatortype, locator):
        try:
            if super().find_elementBy(locatortype, locator):
                element=super().find_elementBy(locatortype, locator)
                actions = TouchAction(self.driver)
                actions.tap(element)
                actions.perform()
        except Exception as error:
            raise error
    '''Double tap on the touch screen using finger motion events
    ''' 
    def double_tap_element(self,locatortype, locator):
        try:
            if super().find_elementBy(locatortype, locator):
                element=super().find_elementBy(locatortype, locator)
                actions = TouchAction(self.driver)
                actions.tap(element,count=2)
                actions.perform()
        except Exception as error:
            raise error 
    
    '''Finger down/press on the screen'''  
    def finger_press_and_move(self,locatortype, locator,x_offset:int,y_offset:int):
        try:
            if super().find_elementBy(locatortype, locator):
                element=super().find_elementBy(locatortype, locator)
                actions = TouchAction(self.driver)
                actions.press(element)
                actions.wait(2)
                actions.move_to(x=x_offset, y=y_offset)
                actions.wait(1)
                actions.perform()
        except Exception as error:
            raise error
    '''Finger up on the screen''' 
    def finger_press_move_Release(self,locatortype, locator,x_offset:int,y_offset:int):
        try:
            if super().find_elementBy(locatortype, locator):
                element=super().find_elementBy(locatortype, locator)
                actions = TouchAction(self.driver)
                actions.press(element)
                actions.wait(2)
                actions.move_to(x=x_offset, y=y_offset)
                actions.wait(2)
                actions.release()
                actions.perform()
        except Exception as error:
            raise error
    '''Long press on the touch screen using finger motion events
    ''' 
    def finger_long_Press(self,locatortype, locator,duration:int):
        try:
            if super().find_elementBy(locatortype, locator):
                element=super().find_elementBy(locatortype, locator)
                actions = TouchAction(self.driver)
                actions.long_press(element,duration=duration)
                actions.perform()
        except Exception as error:
            raise error
    '''Scroll on the touch screen using finger based motion events
    Scrolls from one element to another
    ''' 
    def finger_touch_scroll(self,locatortype, locator_1,locator_2):
        try:
            if super().find_elementBy(locatortype, locator_1) and super().find_elementBy(locatortype, locator_2) :
                element_1=super().find_elementBy(locatortype, locator_1)
                element_2=super().find_elementBy(locatortype, locator_2)
                actions = ActionHelpers(self.driver)
                actions.scroll(element_1,element_2)
                
        except Exception as error:
            raise error
    '''Flick on the touch screen using finger motion events
        Flick from one point to another point.
    ''' 
    def finger_touch_flick(self, start_x: int, start_y: int, end_x: int, end_y: int):
        try:
                actions = ActionHelpers(self.driver)
                actions.flick(start_x,start_y,end_x,end_y)
                
        except Exception as error:
            raise error
    '''Flick on the touch screen using finger motion events
        Flick from one point to another point.
    ''' 
    def finger_Multi_touchAction(self, start_x: int, start_y: int, end_x: int, end_y: int):
        print('TODO') 
    '''Change focus to another window (Web context only)
    ''' 
    def switch_toWindow(self,windowHandle):
        try:
            self.driver.switch_to.window(windowHandle)
        except Exception as error:
            raise error 
    '''Close the current window (Web context only)
    ''' 
    def close_window(self):
        try:
            self.driver.close()
        except Exception as error:
            raise error  
    '''Retrieve the current window handle (Web context only)
    '''           
    def get_current_window_handle(self):
        try:
            return self.driver.current_window_handle
        except Exception as error:
            raise error
    '''Retrieve the all window handles (Web context only)
    '''              
    def get_all_window_handles(self):
        try:
            return self.driver.window_handles
        except Exception as error:
            raise error 
    '''Get the current page title (Web context only)
    '''              
    def get_title(self):
        try:
            return self.driver.title
        except Exception as error:
            raise error 
    '''Get the size of the specified window (Web context only)
    '''
    def get_windowSize_Dimensions(self,windowHandle=None):
        try:
            if (windowHandle !=None):
                return self.driver.get_window_size(windowHandle)
            else:
                return self.driver.get_window_size()   
        except Exception as error:
            raise error
    '''set the size of the current window (Web context only)
    '''
    def set_windowSize_Dimensions(self,width,height,windowHandle: str = 'current'):
        try:

                self.driver.set_window_size(width,height,windowHandle)
  
        except Exception as error:
            raise error 
    '''Get the position of the specified window (Web context only)
    '''
    def get_window_position(self,windowHandle=None):
        try:
            if (windowHandle !=None):
                return self.driver.get_window_position(windowHandle)
            else:
                return self.driver.get_window_position()   
        except Exception as error:
            raise error 
    '''Change the position of the specified window (Web context only)
    '''
    def set_window_position(self,x,y,windowHandle: str = 'current'):
        try:
            self.driver.set_window_position(x,y,windowHandle)
  
        except Exception as error:
            raise error  
    '''Maximize the specified window (Web context only)
    '''
    def maximize_window(self):
        try:
            self.driver.maximize_window()
  
        except Exception as error:
            raise error
    '''Navigate to a new URL (Web context) or open an Appium deep link (Native)
    ''' 
    def goto_url(self,url:str):
        try:
            self.driver.get(url)
  
        except Exception as error:
            raise error
    '''Navigate to a new URL (Web context) or open an Appium deep link (Native)
    ''' 
    def get_current_url(self):
        try:
            return self.driver.current_url
  
        except Exception as error:
            raise error
    '''Navigate backwards in the browser history, if possible (Web context only'''
    def navigate_back(self):
        try:
            return self.driver.back()
  
        except Exception as error:
            raise error 
    '''Navigate forwards in the browser history, if possible (Web context only'''
    def navigate_forward(self):
        try:
            return self.driver.forward()
  
        except Exception as error:
            raise error 
    '''Refresh the current page. (Web context only)
    '''                                                   
    def refresh_current_page(self):
        try:
            return self.driver.refresh()
  
        except Exception as error:
            raise error
    '''Retrieve all cookies visible to the current page (Web context only)
    '''                                                   
    def get_cookies(self):
        try:
            return self.driver.get_cookies()
  
        except Exception as error:
            raise error
    '''Set a cookie (Web context only)
    '''                                                   
    def set_cookies(self,cookie:Dict[str,str]):
        try:
             self.driver.add_cookie()
  
        except Exception as error:
            raise error
    '''Delete the cookie with the given name (Web context only)''' 
    def delete_cookie(self,cookieName:str):
        try:
            self.driver.delete_cookie(cookieName)

  
        except Exception as error:
            raise error 
    '''Delete all cookies visible to current page (Web context only)'''  
    def delete_All_cookie(self):
        try:
            self.driver.delete_all_cookies()

  
        except Exception as error:
            raise error
    '''Change focus to another frame on the page (Web context only)
    Switches focus to the specified frame, by index, name, or webelement.

        :Args:

        frame_reference: The name of the window to switch to, an integer representing the index, or a webelement that is an (i)frame to switch to.
        :Usage:
            :

        driver.switch_to.frame('frame_name')
        driver.switch_to.frame(1)
        driver.switch_to.frame(driver.find_elements(By.TAG_NAME, "iframe")[0])
    '''
    def switchTo_Frame(self,frame_reference: Any):
        try:
            self.driver.switch_to.frame(frame_reference)
  
        except Exception as error:
            raise error 
    '''Change focus to the parent context (Web context only)''' 
    def switch_to_parent_Frame(self):
        try:
            self.driver.switch_to.parent_frame()
  
        except Exception as error:
            raise error
    '''Inject a snippet of JavaScript into the page for execution in the context of the currently 
    selected frame (Web context only''' 
    def driver_execute_async_script(self,script):
        try:
            self.driver.execute_async_script(script)
  
        except Exception as error:
            raise error
    '''Inject a snippet of JavaScript into the page for execution in the context of the currently 
    selected frame (Web context). Run a native mobile command (Native Context).
    ex:self.driver.execute_script("mobile: scroll", {'direction': 'down'})
   ''' 
    def driver_execute_script(self,script):
        try:
            self.driver.execute_script(script)
  
        except Exception as error:
            raise error                                                                   



from mailbox import Mailbox
from re import A
import cv2
import numpy as np
import easyocr
import matplotlib.pyplot as pl
from PIL import Image, ImageChops, ImageDraw
from pyallied.web.webWaits import customwebDriverwait
import textract
import os
import slate3k
import pandas as pd
import PyPDF2
from imap_tools import MailBox
from imap_tools import AND, OR
from datetime import date
from base64 import b64decode
import codecs


class miscellaneous(customwebDriverwait):
    def __init__(self, driver):
        super().__init__(driver)
        self.driver = driver
    '''
    returns the embedded text in the image.
    It returns a list of detected text, with each text element containing three types of information. 
    Which are: the text, its bounding box vertices, and the confidence level of the text detection
    '''

    def get_embeddedText_from_image(self, path: str):
        try:
            reader = easyocr.Reader(['en'])
            textList = []
            data = reader.readtext(path)
            for i in data:
                textList.append(i[1])
            #print(type(data),"---obj trype---")
            # return reader.readtext(path)
            return textList
        except Exception as error:
            raise error

    def is_images_same_compare_by_pixel(self, sourceFile, destinationFile):
        try:
            diff_img = ImageChops.difference(Image.open(
                sourceFile), Image.open(destinationFile)).convert('RGB')
            if diff_img.getbbox():
                return False
            else:
                return True
        except Exception as error:
            raise error

    def new_gray(self,size, color):
        img = Image.new('L', size)
        dr = ImageDraw.Draw(img)
        dr.rectangle((0, 0) + size, color)
        return img

    def marking_differences_With_Original_images(self, image_1, image_2, opacity=0.85):
        try:
            img1 = Image.open(
                image_1)
            img2 = Image.open(
                image_2)
            diff = ImageChops.difference(img1, img2)
            diff = diff.convert('L')
            # Hack: there is no threshold in PILL,
            # so we add the difference with itself to do
            # a poor man's thresholding of the mask:
            # (the values for equal pixels-  0 - don't add up)
            thresholded_diff = diff
            for repeat in range(3):
                thresholded_diff = ImageChops.add(
                    thresholded_diff, thresholded_diff)
            h, w = size = diff.size
            mask = self.new_gray(size, int(255 * (opacity)))
            shade = self.new_gray(size, 0)
            new = img1.copy()
            new.paste(shade, mask=mask)
            # To have the original image show partially
            # on the final result, simply put "diff" instead of thresholded_diff bellow
            new.paste(img2, mask=thresholded_diff)
            return new
        except Exception as error:
            raise error

    def pdf_content_reader(self, file1):
        try:
            file = open(file1, 'rb')

            pdfRead = PyPDF2.PdfFileReader(file)
            #print(pdfRead.getDocumentInfo().title,' ----title----')
            output = []
            readData = pdfRead.getPage(0)
            #print(readData.extractText(),' ----single----')
            for page in pdfRead.pages:
                output.append(page.extractText().replace("\n", " "))
            file.close()
            return output
        except Exception as error:
            raise error

    def mail_outlook_search(self, host, port: int, userName, password, searchString: str, searchDate: str = None):
        try:
            mailbox = MailBox(host, port)
            mailbox.login(userName, password)
            #responses =mailbox.idle.wait(timeout=60)
            mailData = {}
            path = ''
            fileName = ''
            #file2 = open(r"D:/Users/sjyothi/Repos/pythonseleniumFramework/testdata/MyFile2.txt", "w+")
            # data=''
            # if responses:
            # print(responses,'-----respon')
            # for msg in mailbox.fetch():
            if(searchDate != None):
                spliData = searchDate.split("/")
                year = int(spliData[0])
                month = int(spliData[1])
                day = int(spliData[2])
                for msg in mailbox.fetch(AND(date=date(year, month, day))):
                    if(msg.subject in searchString):
                        # print(msg.attachments,'--attach---')
                        if(msg.attachments):
                            # print(msg.date.strftime,"----------------yes--------")
                            cwd = os.getcwd()
                            if(not os.path.exists(cwd+"/tmp")):
                                path = os.mkdir(cwd+"/tmp")
                                # print(path,"---path---")
                            else:
                                path = cwd+"/tmp"
                            for att in msg.attachments:
                                # print(pd.DataFrame(data=att.payload),"-----------------------------------")

                                with open(path+'/{}'.format(att.filename), 'wb') as f:
                                    # print('----',att.filename)
                                    fileName = att.filename
                                    f.write(att.payload)

                                mailData[msg.date_str] = msg.text.replace("\r\n", " "), msg.subject, self.pdf_content_reader(
                                    path+"/"+fileName)
                        else:
                            #print(msg.date_str.replace(" ",""),"----------------yes--------")

                            mailData[msg.date_str] = msg.text.replace(
                                "\r\n", " "), msg.subject
                        # break
                # self.pdf_content_reader(path+"/"+fileName)
            else:
                searchDate = date.today()
                Year = date.today().year
                month = date.today().month
                day = date.today().day
                for msg in mailbox.fetch(AND(date=date(Year, month, day))):
                    if(msg.subject in searchString):
                        # print(msg.attachments,'--attach---')
                        if(msg.attachments):
                            # print(msg.date.strftime,"----------------yes--------")
                            cwd = os.getcwd()
                            if(not os.path.exists(cwd+"/tmp")):
                                path = os.mkdir(cwd+"/tmp")
                                # print(path,"---path---")
                            else:
                                path = cwd+"/tmp"
                            for att in msg.attachments:
                                # print(pd.DataFrame(data=att.payload),"-----------------------------------")

                                with open(path+'/{}'.format(att.filename), 'wb') as f:
                                    # print('----',att.filename)
                                    fileName = att.filename
                                    f.write(att.payload)

                                mailData[msg.date_str] = msg.text.replace("\r\n", " "), msg.subject, self.pdf_content_reader(
                                    path+"/"+fileName)
                        else:
                            # print(msg.date.time+"_"+msg.date.today,"----------------yes--------")

                            mailData[msg.date_str] = msg.text.replace(
                                "\r\n", " "), msg.subject
            mailbox.logout()
            return mailData

        except Exception as error:
            raise error

    def mail_outlook_Attachments(self, host, port: int, userName, password, searchString: str, fromAddress: str):
        try:
            mailbox = MailBox(host, port)
            mailbox.login(userName, password, 'INBOX')
            responses = mailbox.idle.wait(timeout=120)
            attachment = []
            if responses:
                for msg in mailbox.fetch(AND(text=searchString, new=True, from_=fromAddress)):
                    for attach in msg.attachments:
                        attachment.append(attach.filename)
            return attachment
        except Exception as error:
            raise error

    def mail_outlook_Attachments_Read(self, host, port: int, userName, password, searchString: str, fromAddress: str):
        try:
            mailbox = MailBox(host, port)
            mailbox.login(userName, password, 'INBOX')
            responses = mailbox.idle.wait(timeout=120)
            attachment_Read = {}
            if responses:
                for msg in mailbox.fetch(AND(text=searchString, new=True, from_=fromAddress)):
                    for attach in msg.attachments:
                        attachment_Read[attach.filename] = attach.payload
                        # attachment_Read.append(attach.payload)
            return attachment_Read
        except Exception as error:
            raise error
